# -*- coding: utf-8 -*-

"""
    This radish extension provides the functionality to write the feature file run to the console.
"""

from __future__ import unicode_literals
from __future__ import print_function

# disable no-member lint error because of dynamic method from colorful
# pylint: disable=no-member

import os
import re
import colorful

from radish.terrain import world
from radish.hookregistry import before, after
from radish.feature import Feature
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step
from radish.extensionregistry import extension
from radish.utils import console_write as write


@extension
class ConsoleWriter(object):
    """
        Console writer radish extension
    """
    OPTIONS = [
        ("--no-ansi", "print features without any ANSI sequences (like colors, line jump)"),
        ("--no-line-jump", "print features without line jumps (overwriting steps)"),
        ("--write-steps-once", "does not rewrite the steps (this option only makes sense in combination with the --no-ansi flag)"),
        ("--write-ids", "write the feature, scenario and step id before the sentences")
    ]
    LOAD_IF = staticmethod(lambda config: True)
    LOAD_PRIORITY = 30

    def __init__(self):
        before.each_feature(self.console_writer_before_each_feature)
        before.each_scenario(self.console_writer_before_each_scenario)
        before.each_step(self.console_writer_before_each_step)
        after.each_feature(self.console_writer_after_each_feature)
        after.each_scenario(self.console_writer_after_each_scenario)
        after.each_step(self.console_writer_after_each_step)

        self.last_precondition = None
        self.last_background = None

        self._placeholder_regex = re.compile(r'(<[\w-]+>)', flags=re.UNICODE)

    def get_color_func(self, state):
        """
            Returns the color func to use
        """
        if state == Step.State.PASSED:
            return colorful.bold_green
        elif state == Step.State.FAILED:
            return colorful.bold_red
        elif state == Step.State.PENDING:
            return colorful.bold_yellow
        elif state:
            return colorful.cyan

    def get_line_jump_seq(self):
        """
            Returns the line jump ANSI sequence
        """
        line_jump_seq = ""
        if not world.config.no_ansi and not world.config.no_line_jump and not world.config.write_steps_once:
            line_jump_seq = "\r\033[A\033[K"
        return line_jump_seq

    def get_id_sentence_prefix(self, model, color_func, max_rows=None):
        """
            Returns the id from a model as sentence prefix

            :param Model model: a model with an id property
            :param function color_func: a function which gives coloring
            :param int max_rows: the maximum rows. Used for padding
        """
        padding = len("{0}. ".format(max_rows)) if max_rows else 0
        return color_func("{1: >{0}}. ".format(padding, model.id)) if world.config.write_ids else ""

    def get_id_padding(self, max_rows, offset=0):
        """
            Returns the id padding
        """
        if not world.config.write_ids:
            return ""

        return " " * (len(str(max_rows)) + 2 + offset)

    def get_table_col_widths(self, table):
        """
            Returns the width for every column of a table (lists in list)
        """
        return [max(len(str(col)) for col in row) for row in zip(*table)]  # pylint: disable=star-args

    def console_writer_before_each_feature(self, feature):
        """
            Writes feature header to the console

            :param Feature feature: the feature to write to the console
        """
        output = ""
        for tag in feature.tags:
            output += colorful.cyan(u"@{0}{1}\n".format(tag.name, "({0})".format(tag.arg) if tag.arg else ""))

        leading = "\n    " if feature.description else ""

        output += u"{0}{1}: {2}  # {3}{4}{5}".format(
            self.get_id_sentence_prefix(feature, colorful.bold_cyan),
            colorful.bold_white(feature.keyword),
            colorful.bold_white(feature.sentence),
            colorful.bold_black(feature.path),
            leading,
            colorful.white("\n    ".join(feature.description))
        )

        if feature.background:
            output += u"\n\n    {0}: {1}".format(
                colorful.bold_white(feature.background.keyword),
                colorful.bold_white(feature.background.sentence)
            )
            for step in feature.background.all_steps:
                output += '\n' + self._get_step_before_output(step, colorful.cyan)

        write(output)

    def console_writer_before_each_scenario(self, scenario):
        """
            Writes the scenario header to the console

            :param Scenario scenario: the scenario to write to the console
        """
        output = "\n"
        if isinstance(scenario.parent, ScenarioOutline):
            if world.config.write_steps_once:
                return

            id_prefix = self.get_id_sentence_prefix(scenario, colorful.bold_yellow, len(scenario.parent.scenarios))
            colored_pipe = colorful.bold_white("|")
            output = u"        {0}{1} {2} {1}".format(
                id_prefix,
                colored_pipe,
                (u" {0} ").format(colored_pipe).join(
                    str(colorful.bold_yellow(u"{1: <{0}}".format(scenario.parent.get_column_width(i), x))) for i, x in enumerate(scenario.example.data)
                )
            )
        elif isinstance(scenario.parent, ScenarioLoop):
            if world.config.write_steps_once:
                return

            id_prefix = self.get_id_sentence_prefix(scenario, colorful.bold_yellow, len(scenario.parent.scenarios))
            colored_pipe = colorful.bold_white("|")
            output = u"        {0}{1} {2: <18} {1}".format(id_prefix, colored_pipe, str(colorful.bold_yellow(scenario.iteration)))
        else:
            id_prefix = self.get_id_sentence_prefix(scenario, colorful.bold_cyan)
            for tag in scenario.tags:
                if tag.name == "precondition" and world.config.expand and world.config.show:  # exceptional for show command when scenario steps expand and tag is a precondition -> comment it out
                    output += colorful.white(u"    # @{0}{1}\n".format(tag.name, "({0})".format(tag.arg) if tag.arg else ""))
                else:
                    output += colorful.cyan(u"    @{0}{1}\n".format(tag.name, u"({0})".format(tag.arg) if tag.arg else ""))
            output += u"    {0}{1}: {2}".format(id_prefix, colorful.bold_white(scenario.keyword), colorful.bold_white(scenario.sentence))
        write(output)

    def console_writer_before_each_step(self, step):
        """
            Writes the step to the console before it is run

            :param Step step: the step to write to the console
        """
        if not isinstance(step.parent.parent, Feature):
            return

        if world.config.write_steps_once:
            return

        output = ""
        if step.as_precondition and self.last_precondition != step.as_precondition:
            if step.as_background:
                output += colorful.italic_white(u"      As Background Precondition from {0}: {1}\n".format(os.path.basename(step.as_precondition.path), step.as_precondition.sentence))
            else:
                output += colorful.italic_white(u"      As Precondition from {0}: {1}\n".format(os.path.basename(step.as_precondition.path), step.as_precondition.sentence))
        elif step.as_background and self.last_background != step.as_background:
            output += colorful.italic_white(u"      From Background: {0}\n".format(step.as_background.sentence))
        elif step.as_precondition and self.last_precondition and not step.as_background and self.last_background:
            output += colorful.italic_white(u"      From Precondition Scenario: {0}: {1}\n".format(os.path.basename(step.as_precondition.path), step.as_precondition.sentence))
        elif (not step.as_precondition and self.last_precondition) or (not step.as_background and self.last_background):
            output += colorful.italic_white(u"      From Scenario\n")

        self.last_precondition = step.as_precondition
        self.last_background = step.as_background
        output += self._get_step_before_output(step)

        write(output)

    def _get_step_before_output(self, step, color_func=None):
        if color_func is None:
            color_func = colorful.bold_yellow
        output = u"\r        {0}{1}".format(self.get_id_sentence_prefix(step, color_func), color_func(step.sentence))

        if step.text:
            id_padding = self.get_id_padding(len(step.parent.steps))
            output += colorful.bold_white(u'\n            {0}"""'.format(id_padding))
            output += colorful.cyan(u"".join(["\n                {0}{1}".format(id_padding, l) for l in step.raw_text]))
            output += colorful.bold_white(u'\n            {0}"""'.format(id_padding))

        if step.table_header:
            colored_pipe = colorful.bold_white("|")
            col_widths = self.get_table_col_widths([step.table_header] + step.table_data)

            # output table header
            output += u"\n          {0} {1} {0}".format(colored_pipe, (" {0} ").format(colored_pipe).join(
                str(colorful.white(u"{1: <{0}}".format(col_widths[i], x))) for i, x in enumerate(step.table_header)
            ))

            # output table data
            for row in step.table_data:
                output += u"\n          {0} {1} {0}".format(colored_pipe, (" {0} ").format(colored_pipe).join(
                    str(color_func(u"{1: <{0}}".format(col_widths[i], x))) for i, x in enumerate(row)
                ))

        return output


    def console_writer_after_each_step(self, step):
        """
            Writes the step to the console after it was run

            :param Step step: the step to write to the console
        """
        if not isinstance(step.parent.parent, Feature):
            return

        color_func = self.get_color_func(step.state)
        line_jump_seq = self.get_line_jump_seq() * (((len(step.raw_text) + 3) if step.text else 1) + (len(step.table) + 1 if step.table_header else 0))
        output = u'{0}        '.format(line_jump_seq)

        if isinstance(step.parent, ScenarioOutline):
            # Highlight ScenarioOutline placeholders e.g. '<method>'
            output += (u''.join(str(colorful.white(item) if (self._placeholder_regex.search(item)
                                and item.strip('<>') in step.parent.examples_header)
                                else color_func(item))
                                for item in self._placeholder_regex.split(step.sentence)))
        else:
            output += u"{0}{1}".format(self.get_id_sentence_prefix(step, colorful.bold_cyan), color_func(step.sentence))

        if step.text:
            id_padding = self.get_id_padding(len(step.parent.steps))
            output += colorful.bold_white(u'\n            {0}"""'.format(id_padding))
            output += colorful.cyan(u"".join(["\n                {0}{1}".format(id_padding, l) for l in step.raw_text]))
            output += colorful.bold_white(u'\n            {0}"""'.format(id_padding))

        if step.table_header:
            colored_pipe = colorful.bold_white("|")
            col_widths = self.get_table_col_widths([step.table_header] + step.table_data)

            # output table header
            output += u"\n          {0} {1} {0}".format(colored_pipe, (" {0} ").format(colored_pipe).join(
                str(colorful.white(u"{1: <{0}}".format(col_widths[i], x))) for i, x in enumerate(step.table_header)
            ))

            # output table data
            for row in step.table_data:
                output += u"\n          {0} {1} {0}".format(colored_pipe, (" {0} ").format(colored_pipe).join(
                    str(color_func(u"{1: <{0}}".format(col_widths[i], x))) for i, x in enumerate(row)
                ))

        if step.state == step.State.FAILED:
            if world.config.with_traceback:
                output += u"\n          {0}{1}".format(self.get_id_padding(len(step.parent.steps) - 2), "\n          ".join([str(colorful.red(l)) for l in step.failure.traceback.split("\n")[:-2]]))
            output += u"\n          {0}{1}: {2}".format(self.get_id_padding(len(step.parent.steps) - 2), colorful.bold_red(step.failure.name), colorful.red(step.failure.reason))

        write(output)

    def console_writer_after_each_scenario(self, scenario):
        """
            If the scenario is a ExampleScenario it will write the Examples header

            :param Scenario scenario: the scenario which was ran.
        """
        output = ""
        if isinstance(scenario, ScenarioOutline):
            output += u"\n    {0}:\n".format(colorful.bold_white(scenario.example_keyword))
            output += colorful.bold_white(u"        {0}| {1} |".format(
                self.get_id_padding(len(scenario.scenarios), offset=2),
                u" | ".join("{1: <{0}}".format(scenario.get_column_width(i), x) for i, x in enumerate(scenario.examples_header))
            ))
        elif isinstance(scenario, ScenarioLoop):
            output += u"\n    {0}: {1}".format(colorful.bold_white(scenario.iterations_keyword), colorful.cyan(scenario.iterations))
        elif isinstance(scenario.parent, ScenarioOutline):
            colored_pipe = colorful.bold_white("|")
            color_func = self.get_color_func(scenario.state)
            output += u"{0}        {1}{2} {3} {2}".format(
                self.get_line_jump_seq(),
                self.get_id_sentence_prefix(scenario, colorful.bold_cyan, len(scenario.parent.scenarios)),
                colored_pipe,
                (u" {0} ").format(colored_pipe).join(
                    str(color_func(u"{1: <{0}}".format(scenario.parent.get_column_width(i), x))) for i, x in enumerate(scenario.example.data)
                )
            )

            if scenario.state == Step.State.FAILED:
                failed_step = scenario.failed_step
                if world.config.with_traceback:
                    output += u"\n          {0}{1}".format(self.get_id_padding(len(scenario.parent.scenarios)), "\n          ".join([str(colorful.red(l)) for l in failed_step.failure.traceback.split("\n")[:-2]]))
                output += u"\n          {0}{1}: {2}".format(self.get_id_padding(len(scenario.parent.scenarios)), colorful.bold_red(failed_step.failure.name), colorful.red(failed_step.failure.reason))
        elif isinstance(scenario.parent, ScenarioLoop):
            colored_pipe = colorful.bold_white("|")
            color_func = self.get_color_func(scenario.state)
            output += u"{0}        {1}{2} {3: <18} {2}".format(self.get_line_jump_seq(), self.get_id_sentence_prefix(scenario, colorful.bold_cyan, len(scenario.parent.scenarios)), colored_pipe, str(color_func(scenario.iteration)))

            if scenario.state == Step.State.FAILED:
                failed_step = scenario.failed_step
                if world.config.with_traceback:
                    output += u"\n          {0}{1}".format(self.get_id_padding(len(scenario.parent.scenarios)), "\n          ".join([str(colorful.red(l)) for l in failed_step.failure.traceback.split("\n")[:-2]]))
                output += u"\n          {0}{1}: {2}".format(self.get_id_padding(len(scenario.parent.scenarios)), colorful.bold_red(failed_step.failure.name), colorful.red(failed_step.failure.reason))

        if output:
            write(output)

    def console_writer_after_each_feature(self, feature):  # pylint: disable=unused-argument
        """
            Writes a newline after each feature

            :param Feature feature: the feature which was ran.
        """
        write("")



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

import radish.utils as utils


@pytest.mark.parametrize('basedirs, expected_basedirs', [
    (['foo', 'bar'], ['foo', 'bar']),
    (['foo:bar', 'foobar'], ['foo', 'bar', 'foobar']),
    (['foo:bar', 'foobar', 'one:two:three'],
        ['foo', 'bar', 'foobar', 'one', 'two', 'three']),
    (['foo:', ':bar'], ['foo', 'bar'])
])
def test_flattened_basedirs(basedirs, expected_basedirs):
    """
    Test flatten basedirs
    """
    # given & when
    actual_basedirs = utils.flattened_basedirs(basedirs)

    # then
    assert actual_basedirs == expected_basedirs



# -*- coding: utf-8 -*-

from radish import before
from radish import world
from radish import custom_type


@before.each_scenario
def setup_hero_db(scenario):
    world.heros = []


@custom_type("Hero", r"[A-Z][a-z]+")
def get_hero_custom_type(text):
    """
    Return a hero object by the given name
    """
    for hero in world.heros:
        if hero.heroname == text:
            return hero

    return None



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.feature import Feature
from radish.scenario import Scenario
from radish.background import Background
from radish.stepmodel import Step


def test_creating_simple_scenario():
    """
    Test creating a simple Scenario
    """
    # given & when
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=None, background=None)

    # then
    assert scenario.id == 1
    assert scenario.keyword == 'Scenario'
    assert scenario.sentence == 'I am a Scenario'
    assert scenario.path == 'foo.feature'
    assert scenario.line == 1
    assert scenario.parent is None
    assert scenario.tags == []
    assert scenario.preconditions == []
    assert scenario.background is None


def test_scenario_state(mocker):
    """
    Test getting the Scenario state according to it's Steps states
    """
    # given
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=None, background=None)
    # add Steps to this Scenario
    scenario.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # get the step to modify
    step = scenario.steps[1]

    # when all Steps are passed the Scenario is passed
    assert scenario.state == Step.State.PASSED

    # when one Step is failed the Scenario is failed
    step.state = Step.State.FAILED
    assert scenario.state == Step.State.FAILED

    # when one Step is pending the Scenario is pending
    step.state = Step.State.PENDING
    assert scenario.state == Step.State.PENDING

    # when one Step is skipped the Scenario is skipped
    step.state = Step.State.SKIPPED
    assert scenario.state == Step.State.SKIPPED

    # when one Step is untested the Scenario is untested
    step.state = Step.State.UNTESTED
    assert scenario.state == Step.State.UNTESTED


def test_scenario_state_with_background(mocker):
    """
    Test getting the Scenario state according to it's Steps states including a Background
    """
    # given
    background = mocker.MagicMock(steps=[])
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=None, background=background)
    # add Steps to this Scenario
    scenario.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # add Steps to the background
    background.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # get the step to modify
    step = background.steps[1]

    # when all Steps are passed the Scenario is passed
    assert scenario.state == Step.State.PASSED

    # when a Background Step is failed the Scenario is failed
    step.state = Step.State.FAILED
    assert scenario.state == Step.State.FAILED


def test_scenario_all_steps(mocker):
    """
    Test getting all Steps which are part of a Scenario
    """
    # given
    background = mocker.MagicMock(all_steps=[])
    precondition_scenario = mocker.MagicMock(all_steps=[])
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=[precondition_scenario], background=background)

    # when
    # add Steps to this Scenario
    scenario.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # add Steps to the Background
    background.all_steps.extend([
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # add Steps to the precondition Scenario
    precondition_scenario.all_steps.extend([
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED)
    ])

    # then
    assert len(scenario.all_steps) == 6


def test_get_scenario_constants():
    """
    Test getting all constants from a Scenario
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 2, parent=feature,
                        tags=None, preconditions=None, background=None)
    # add Feature constants
    feature.context.constants = [('foo', '1'), ('bar', '42')]
    # add Scenario constants
    scenario.context.constants = [('some_foo', '${foo}3'), ('answer', '${bar}')]

    # when
    constants = scenario.constants

    assert len(constants) == 4
    assert constants[0] == ('some_foo', '13')
    assert constants[1] == ('answer', '42')
    assert constants[2] == ('foo', '1')
    assert constants[3] == ('bar', '42')


def test_scenario_failed_step(mocker):
    """
    Test getting the first failed Step from a Scenario
    """
    # given
    background = mocker.MagicMock(steps=[])
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=None, background=background)

    # when
    # add Steps to this Scenario
    scenario.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED)
    ])
    # add Steps to the Background
    background.steps.extend([
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED),
        mocker.MagicMock(state=Step.State.PASSED)
    ])

    # when no Step failed
    assert scenario.failed_step is None

    # when a Scenario Step fails it should be returned
    scenario.steps[0].state = Step.State.FAILED
    assert scenario.failed_step == scenario.steps[0]
    scenario.steps[0].state = Step.State.PASSED

    # when a Background Step fails it should be returned
    background.steps[0].state = Step.State.FAILED
    assert scenario.failed_step == background.steps[0]
    background.steps[0].state = Step.State.PASSED

    # when a Background and a Scenario Step fails the
    # Background Step should be returned
    background.steps[0].state = Step.State.FAILED
    scenario.steps[0].state = Step.State.FAILED
    assert scenario.failed_step == background.steps[0]


@pytest.mark.parametrize('scenario_id, scenario_choice, expected_has_to_run', [
    (1, [], True),
    (1, [1], True),
    (2, [1, 2, 5], True),
    (1, [4], False),
    (1, [4, 5], False),
])
def test_scenario_has_to_run(scenario_id, scenario_choice, expected_has_to_run):
    """
    Test logic to check whether a Scenario has to run or not
    """
    # given
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=None, background=None)
    scenario.absolute_id = scenario_id

    # when
    actual_has_to_run = scenario.has_to_run(scenario_choice)

    # then
    assert actual_has_to_run is expected_has_to_run


def test_scenario_after_parse_logic(mocker):
    """
    Test logic which is used to complete the parsing of Scenario
    """
    # given
    background = Background(1, 'Background', 'I am a Background', 'foo.feature', 1)
    precondition_scenario = Scenario(2, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None)
    scenario = Scenario(1, 'Scenario', 'I am a Scenario', 'foo.feature', 1, parent=None,
                        tags=None, preconditions=[precondition_scenario], background=background)
    # add Steps to this Scenario
    scenario.steps.extend([
        mocker.MagicMock(id=99, as_background=False, as_precondition=False),
    ])
    # set Scenario Step parents
    for step in scenario.steps:
        step.parent = scenario
    # add Steps to the Background
    background.steps.extend([
        mocker.MagicMock(id=5, as_background=False, as_precondition=False),
        mocker.MagicMock(id=6, as_background=False, as_precondition=False),
        mocker.MagicMock(id=66, as_background=False, as_precondition=False)
    ])
    # set Background Step parents
    for background_step in background.steps:
        background_step.parent = background
    # add Steps to the precondition Scenario
    precondition_scenario.steps.extend([
        mocker.MagicMock(id=5, as_background=False, as_precondition=False),
        mocker.MagicMock(id=77, as_background=False, as_precondition=False)
    ])
    # set Precondition Scenario Step parents
    for step in precondition_scenario.steps:
        step.parent = precondition_scenario

    # when after_parse() was not called it's not completed
    assert scenario.complete is False

    # when
    scenario.after_parse()
    steps = scenario.all_steps

    # then - the Scenario is completed
    assert scenario.complete is True

    # then - the step id's are in valid order
    assert steps[0].id == 1
    assert steps[1].id == 2
    assert steps[2].id == 3
    assert steps[3].id == 4
    assert steps[4].id == 5
    assert steps[5].id == 6

    # then - check as_background flags
    assert all(step.as_background for step in background.steps)
    # then - check as_precondition flags
    assert all(step.as_precondition for step in precondition_scenario.steps)



# -*- coding: utf-8 -*-

"""
This module provides a radish extension to make
coverage measurements.
"""

import sys
import re

from radish.extensionregistry import extension
from radish.hookregistry import before, after
from radish.terrain import world
from radish.compat import StringIO
from radish.exceptions import RadishError


@extension
class CodeCoverage(object):
    """
        Code Coverage radish extension
    """
    OPTIONS = [
        ('--with-coverage', 'enable code coverage'),
        ('--cover-packages=<cover_packages>', 'specify source code package'),
        ('--cover-append', 'append coverage data to previous collected data'),
        ('--cover-config-file=<cover_config_file>', 'specify coverage config file [default: .coveragerc]'),
        ('--cover-branches', 'include branch coverage in report'),
        ('--cover-erase', 'erase previously collected coverage data'),
        ('--cover-min-percentage=<cover_min_percentage>', 'fail if the given minimum coverage percentage is not reached'),
        ('--cover-html=<cover_html_dir>', 'specify a directory where to store HTML coverage report'),
        ('--cover-xml=<cover_xml_file>', 'specify a file where to store XML coverage report')
    ]
    LOAD_IF = staticmethod(lambda config: config.with_coverage)
    LOAD_PRIORITY = 70

    def __init__(self):
        try:
            from coverage import Coverage
        except ImportError:
            raise RadishError('if you want to use the code coverage you have to "pip install radish-bdd[coverage]"')

        before.all(self.coverage_start)
        after.all(self.coverage_stop)

        if world.config.cover_packages:
            self.cover_packages = world.config.cover_packages.split(",")
        else:
            self.cover_packages = []

        self.coverage = None
        self.modules_on_init = set(sys.modules.keys())

    def coverage_start(self, features, marker):
        """
        Start the coverage measurement
        """
        from coverage import Coverage
        # if no explicit modules are specified we just
        # use the ones loaded from radish's basedir.
        # During the plugin init the basedir modules are
        # not loaded yet, but they are during the start method.
        # Thus, we are safe to consider the difference between the
        # two for coverage measurement.
        if not self.cover_packages:
            source = list(set(sys.modules.keys()).difference(self.modules_on_init))
        else:
            source = self.cover_packages

        self.coverage = Coverage(source=source,
                                 config_file=world.config.cover_config_file,
                                 branch=world.config.cover_branches)
        if world.config.cover_erase:
            self.coverage.combine()
            self.coverage.erase()

        if world.config.cover_append:
            self.coverage.load()
        self.coverage.start()

    def coverage_stop(self, features, marker):
        """
        Stop the coverage measurement
        and create report
        """
        self.coverage.stop()
        self.coverage.combine()
        self.coverage.save()
        self.coverage.report(file=sys.stdout)

        if world.config.cover_html:
            self.coverage.html_report(directory=world.config.cover_html)

        if world.config.cover_xml:
            self.coverage.xml_report(outfile=world.config.cover_xml)

        if world.config.cover_min_percentage:
            report = StringIO()
            self.coverage.report(file=report)
            match = re.search(r'^TOTAL\s+(.*)$', report.getvalue(), re.MULTILINE)
            if not match:
                raise RadishError('Failed to find total percentage in coverage report')

            total_percentage = int(match.groups()[0].split()[-1][:-1])
            if total_percentage < int(world.config.cover_min_percentage):
                raise RadishError('Failed to reach minimum expected coverage of {0}% (reached: {1}%)'.format(
                    world.config.cover_min_percentage, total_percentage))



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import os
import sys
import tempfile

import pytest

from radish.main import main


@pytest.mark.parametrize('given_featurefiles, given_cli_args, expected_exitcode, expected_output', [
    (
        ['empty'], [], 1, 'empty'
    ),
    (
        ['empty-feature'], [], 1, 'empty-feature'
    ),
    (
        ['feature-only-description'], [], 1, 'feature-only-description'
    ),
    (
        ['feature-scenario-steps'], [], 0, 'feature-scenario-steps'
    ),
    (
        ['feature-scenarios'], [], 0, 'feature-scenarios'
    ),
    (
        ['comments'], [], 0, 'comments'
    ),
    (
        ['german'], [], 0, 'german'
    ),
    (
        ['multi-features'], [], 1, 'multi-features'
    ),
    (
        ['empty-scenario'], [], 0, 'empty-scenario'
    ),
    (
        ['scenario-outline'], [], 0, 'scenario-outline'
    ),
    (
        ['regular-scenario-examples'], [], 1, 'regular-scenario-examples'
    ),
    (
        ['scenario-loop'], [], 0, 'scenario-loop'
    ),
    (
        ['step-tabular-data'], [], 0, 'step-tabular-data'
    ),
    (
        ['step-tabular-data-invalid'], [], 1, 'step-tabular-data-invalid'
    ),
    (
        ['step-text-data'], [], 0, 'step-text-data',
    ),
    (
        ['tags-feature'], [], 0, 'tags-feature',
    ),
    (
        ['tags-feature'], ['--tags', 'foo'], 0, 'tags-feature',
    ),
    (
        ['tags-feature'], ['--tags', 'foo or bar'], 0, 'tags-feature',
    ),
    (
        ['tags-feature'], ['--tags', 'foo and bar'], 0, 'tags-feature',
    ),
    (
        ['tags-scenario'], [], 0, 'tags-scenario',
    ),
    (
        ['tags-scenario'], ['--tags', 'foo'], 0, 'tags-scenario',
    ),
    (
        ['tags-scenario'], ['--tags', 'foo or bar'], 0, 'tags-scenario',
    ),
    (
        ['tags-scenario'], ['--tags', 'foo and bar'], 0, 'tags-scenario',
    ),
    (
        ['tags-everywhere'], [], 0, 'tags-everywhere',
    ),
    (
        ['tags-everywhere'], ['--tags', 'regular_scenario'], 0, 'tags-everywhere-regular-scenario-only',
    ),
    (
        ['tags-everywhere'], ['--tags', 'scenario_outline'], 0, 'tags-everywhere-scenario-outline-only',
    ),
    (
        ['tags-everywhere'], ['--tags', 'scenario_loop'], 0, 'tags-everywhere-scenario-loop-only',
    ),
    (
        ['tags-everywhere'], ['--tags', 'scenario_loop or scenario_outline'], 0, 'tags-everywhere-scenario-loop-and-outline',
    ),
    (
        ['tags-arguments'], [], 0, 'tags-arguments'
    ),
    (
        ['tags-ignored-scenario'], [], 0, 'tags-ignored-scenario'
    ),
    (
        ['tags-ignored-scenario'], ['--tags', 'foo'], 0, 'tags-ignored-scenario-foo-only'
    ),
    (
        ['tags-ignored-scenario'], ['--tags', 'bar'], 0, 'tags-ignored-scenario-bar-only'
    ),
    (
        ['tags-ignored-scenario'], ['--tags', 'bar or foo'], 0, 'tags-ignored-scenario-bar-or-foo'
    ),
    (
        ['tags-ignored-scenario'], ['--tags', 'not bar and not foo'], 0, 'tags-ignored-scenario-not-bar-not-foo'
    ),
    (
        ['background'], [], 0, 'background'
    ),
    (
        ['background-scenariooutline'], [], 0, 'background-scenariooutline'
    ),
    (
        ['background-scenarioloop'], [], 0, 'background-scenarioloop'
    ),
    (
        ['background-subsequent-tag'], [], 0, 'background-subsequent-tag'
    ),
    (
        ['background-misplaced'], [], 1, 'background-misplaced'
    ),
    (
        ['background-multiple'], [], 1, 'background-multiple'
    ),
    (
        ['constants'], [], 0, 'constants'
    ),
    (
        ['scenario-sentence-duplicate'], [], 1, 'scenario-sentence-duplicate'
    ),
    (
        ['precondition-level-0'], [], 0, 'precondition-level-0'
    ),
    (
        ['precondition-level-1'], [], 0, 'precondition-level-1'
    ),
    (
        ['precondition-level-2'], [], 0, 'precondition-level-2'
    ),
    (
        ['precondition-same-feature'], [], 0, 'precondition-same-feature'
    ),
    (
        ['precondition-unknown-scenario-same-feature'], [], 1, 'precondition-unknown-scenario-same-feature',
    ),
    (
        ['precondition-malformed'], [], 1, 'precondition-malformed'
    ),
    (
        ['failing-scenario'], [], 1, 'failing-scenario'
    ),
    (
        ['failing-scenario-middle'], [], 1, 'failing-scenario-middle'
    ),
    (
        ['failing-scenario-outline'], [], 1, 'failing-scenario-outline'
    ),
    (
        ['failing-scenario-outline-middle'], [], 1, 'failing-scenario-outline-middle'
    ),
    (
        ['failing-scenario-outline-middle'], ['--early-exit'], 1, 'failing-scenario-outline-middle-exit-early'
    ),
    (
        ['failing-scenario-loop'], [], 1, 'failing-scenario-loop'
    ),
    (
        ['failing-scenario-loop'], ['--early-exit'], 1, 'failing-scenario-loop-exit-early'
    ),
    (
        ['feature-scenario-steps'], ['--write-ids'], 0, 'feature-scenario-steps-with-ids'
    ),
    (
        ['scenario-outline'], ['--write-ids'], 0, 'scenario-outline-with-ids'
    ),
    (
        ['scenario-loop'], ['--write-ids'], 0, 'scenario-loop-with-ids'
    ),
    (
        ['background'], ['--write-ids'], 0, 'background-with-ids'
    ),
    (
        ['feature-scenario-steps'], ['--no-ansi'], 0, 'feature-scenario-steps-no-ansi'
    ),
    (
        ['feature-scenario-steps'], ['--no-ansi', '--write-steps-once'], 0, 'feature-scenario-steps-no-ansi-write-once'
    ),
    (
        ['scenario-outline'], ['--no-ansi'], 0, 'scenario-outline-no-ansi'
    ),
    (
        ['scenario-outline'], ['--no-ansi', '--write-steps-once'], 0, 'scenario-outline-no-ansi-write-once'
    ),
    (
        ['scenario-loop'], ['--no-ansi'], 0, 'scenario-loop-no-ansi'
    ),
    (
        ['scenario-loop'], ['--no-ansi', '--write-steps-once'], 0, 'scenario-loop-no-ansi-write-once'
    ),
    (
        ['feature-scenario-steps'], ['--no-line-jump'], 0, 'feature-scenario-steps-no-line-jump'
    ),
    (
        ['scenario-outline'], ['--no-line-jump'], 0, 'scenario-outline-no-line-jump'
    ),
    (
        ['scenario-loop'], ['--no-line-jump'], 0, 'scenario-loop-no-line-jump'
    ),
    (
        ['feature-scenario-steps'], ['--bdd-xml', tempfile.mkstemp()[1]], 0, 'feature-scenario-steps'
    ),
    (
        ['feature-scenario-steps'], ['--cucumber-json', tempfile.mkstemp()[1]], 0, 'feature-scenario-steps'
    ),
    (
        ['feature-scenario-steps'], ['--junit-xml', tempfile.mkstemp()[1]], 0, 'feature-scenario-steps'
    ),    
], ids=[
    'Empty Feature File',
    'Empty Featre',
    'Feature with description only',
    'Feature with one Scenario and Steps',
    'Feature with multiple Scenarios',
    'Comments in Feature',
    'German Keywords',
    'Multiple Features in single Feature File',
    'Empty Scenario',
    'Scenario Outline',
    'Regular Scenario with Scenario Outline Examples',
    'Scenario Loop',
    'Step with Tabular data',
    'Step Tabular data without previous Step',
    'Step with Text data',
    'Feature with Tags',
    'Feature with Tags filtered by Foo',
    'Feature with Tags filtered by Foo or Bar',
    'Feature with Tags filtered by Foo and Bar',
    'Scenario with Tags',
    'Scenario with Tags filtered by Foo',
    'Scenario with Tags filtered by Foo or Bar',
    'Scenario with Tags filtered by Foo and Bar',
    'Feature and Scenario, Scenario Outline and Loop with Tags',
    'Feature and Scenario, Scenario Outline and Loop with Tags filtered by regular_scenario',
    'Feature and Scenario, Scenario Outline and Loop with Tags filtered by scenario_outline',
    'Feature and Scenario, Scenario Outline and Loop with Tags filtered by scenario_loop',
    'Feature and Scenario, Scenario Outline and Loop with Tags filtered by scenario_loop or scenario_outline',
    'Tag Arguments for Feature and Scenario Tags',
    'Feature and Scenario with Tags to ignore',
    'Only show Scenarios tagged with foo',
    'Only show Scenarios tagged with bar',
    'Only show Scenarios tagged with foo or bar',
    'Only show Scenarios tagged not with foo and bar',
    'Background',
    'Background for Scenario Outline',
    'Background for Scenario Loop',
    'Background with subsequent Tag',
    'Background which is misplaced',
    'Multiple Background in one Feature',
    'Feature and Scenario Constants',
    'Scenario Sentence Duplicate',
    'Precondition Level 0',
    'Precondition Level 1',
    'Precondition Level 2',
    'Precondition from same Feature',
    'Precondition with unknown Scenario from same Feature',
    'Precondition which is malformed',
    'Failing Scenario',
    'Failing Scenario in the middle',
    'Failing Scenario Outline',
    'Failing Scenario Outline in the middle',
    'Failing Scenario Outline in the middle with early exit',
    'Failing Scenario Loop',
    'Failing Scenario Loop with early exit',
    'Feature with single Scenario and Steps with Ids',
    'Scenario Outline with Ids',
    'Scenario Loop with Ids',
    'Background with Ids',
    'Feature with single Scenario and Steps with no ANSI',
    'Feature with single Scenario and Steps with no ANSI and Steps once',
    'Scenario Outline with no ANSI',
    'Scenario Outline with no ANSI and Steps once',
    'Scenario Loop with no ANSI',
    'Scenario Loop with no ANSI and Steps once',
    'Feature with single Scenario and Steps with no line jump',
    'Scenario Outline with no line jump',
    'Scenario Loop with no line jump',
    'Feature with single Scenario and Steps producing BDD XML',
    'Feature with single Scenario and Steps producing Cucumber JSON',
    'Feature with single Scenario and Steps producing JUnit XML',
])
def test_main_cli_calls(given_featurefiles, given_cli_args, expected_exitcode, expected_output,
                        featurefiledir, radishdir, outputdir):
    """
    Test calling main CLI
    """
    # given
    if '-m' not in given_cli_args and '--marker' not in given_cli_args:
        given_cli_args.extend(['--marker', 'test-marker'])

    if '-b' not in given_cli_args and '--basedir' not in given_cli_args:
        given_cli_args.extend(['-b', radishdir])
    else:
        # fixup basedir paths
        base_dir_idx = [i for i, x in enumerate(given_cli_args) if x in ('-b', '--basedir')]
        for idx in base_dir_idx:
            given_cli_args[idx + 1] = os.path.join(radishdir, given_cli_args[idx + 1])

    featurefiles = [os.path.join(featurefiledir, x + '.feature') for x in given_featurefiles]
    cli_args = featurefiles + given_cli_args

    expected_output_file = os.path.join(outputdir, expected_output + '.txt')
    with open(expected_output_file, 'r') as output_file:
        expected_output_string = output_file.read()

    # when
    original_stdout = sys.stdout

    with tempfile.TemporaryFile(mode='w+') as tmp_stdout:
        # patch sys.stdout
        sys.stdout = tmp_stdout

        try:
            actual_exitcode = main(args=cli_args)
        except SystemExit as exc:
            actual_exitcode = exc.code
        finally:
            tmp_stdout.seek(0)
            actual_output = tmp_stdout.read()
            # restore stdout
            sys.stdout = original_stdout

    # patch featurefile paths in actual output
    feature_parent_dir = os.path.abspath(os.path.join(featurefiledir, '..'))
    for featurefile in featurefiles:
        rel_featurefile = os.path.relpath(featurefile, feature_parent_dir)
        actual_output = actual_output.replace(featurefile, rel_featurefile)

    # then
    assert actual_output == expected_output_string
    assert actual_exitcode == expected_exitcode



# -*- coding: utf-8 -*-

"""
    Terrain module providing step overlapping data containers
"""

import threading

world = threading.local()  # pylint: disable=invalid-name


def pick(func):
    """
        Picks the given function and add it to the world object
    """
    setattr(world, func.__name__, func)
    return func


world.pick = pick



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""


from radish import given, when, then


@given('I have a step')
def have_a_step(step):
    "Given I have a step"
    pass


@when('I do something')
def do_something(step):
    "When I do something"
    pass


@then('I expect something')
def expect_something(step):
    "Then I expect something"
    pass


@given('I have the number {number:d}')
def have_number(step, number):
    "Given I have the number <n>"
    if not hasattr(step.context, 'numbers'):
        step.context.numbers = []

    step.context.numbers.append(number)


@when('I add them up')
def sum_numbers(step):
    "When I add them up"
    step.context.sum = sum(step.context.numbers)


@when('I add them up with failure')
def sum_numbers(step):
    "When I add them up with failure"
    assert False, 'Unable to add numbers: {0}'.format(step.context.numbers)


@when('I subtract them')
def subtract_numbers(step):
    "When I subtract them up"
    difference = step.context.numbers[0]
    for n in step.context.numbers[1:]:
        difference -= n
    step.context.difference = difference


@then('I expect the sum to be {expected_sum:d}')
def expect_sum(step, expected_sum):
    "Then I expect the sum to be <n>"
    assert step.context.sum == expected_sum, 'The expected sum {0} does not match actual sum {1}'.format(
        expected_sum, step.context.sum)


@then('I expect the difference to be {expected_diff:d}')
def expect_sum(step, expected_diff):
    "Then I expect the difference to be <n>"
    assert step.context.difference == expected_diff, 'The expected difference {0} does not match actual difference {1}'.format(
        expected_diff. step.context.difference)


@given('I have an instable function')
def have_instable_function(step):
    "Given I have an instable function"
    pass


@when('I execute it')
def execute_instable_function(step):
    "When I execute it"
    pass


@then('I expect it to pass')
def expect_instable_function_pass(step):
    "Then I expect it to pass"
    pass


@given('I have the following heros')
def have_heros(step):
    "Given I have the following heros"
    step.context.heros = step.table


@when('I capitalize their first name')
def cap_first_name(step):
    "When I capitalize their first name"
    for hero in step.context.heros:
        hero['firstname'] = hero['firstname'].upper()


@then('I have the following names')
def have_names(step):
    "Then I have the following names"
    assert list(x['firstname'] for x in step.context.heros) == list(
            x['cap_heroname'] for x in step.table)


@given('I have the following quote')
def have_quote(step):
    "Given I have the following quote"
    step.context.quote = step.text


@when("I look for it's author")
def lookup_author(step):
    "When I look for it's author"
    step.context.author = 'Shakespeare'


@then('I will find {:S}')
def expect_author(step, author):
    "Then I will find <author>"
    assert step.context.author == author



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.feature import Feature
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step


def test_creating_simple_feature():
    """
    Test creating a simple Feature
    """
    # given & when
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)

    # then
    assert feature.id == 1
    assert feature.keyword == 'Feature'
    assert feature.sentence == 'I am a feature'
    assert feature.path == 'foo.feature'
    assert feature.line == 1
    assert feature.tags == []


def test_feature_representation():
    """
    Test Feature representation with str() and repr()
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)

    # when
    str_repr = str(feature)
    repr_repr = repr(feature)

    # then
    assert str_repr == 'Feature: I am a feature from foo.feature:1'
    assert repr_repr == '<Feature: I am a feature from foo.feature:1>'


def test_feature_scenario_iterator(mocker):
    """
    Test iterating over Scenarios within a Feature
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add Scenarios to Feature
    feature.scenarios.extend([
        mocker.MagicMock(id=1), mocker.MagicMock(id=2), mocker.MagicMock(id=3)
    ])

    # when
    scenario_iterator = iter(feature)

    # then
    assert next(scenario_iterator).id == 1
    assert next(scenario_iterator).id == 2
    assert next(scenario_iterator).id == 3

    with pytest.raises(StopIteration):
        next(scenario_iterator)


def test_feature_scenario_iterator_empty():
    """
    Test iterating over Scenarios within a Feature when no Scenarios
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)

    # when
    scenario_iterator = iter(feature)

    # then
    with pytest.raises(StopIteration):
        next(scenario_iterator)


@pytest.mark.parametrize('scenario_sentences, expected_scenario, found', [
    (('foo', 'bar'), 'foo', True),
    (('bar'), 'foo', False),
    ([], 'foo', False)
])
def test_feature_contains_scenario(scenario_sentences, expected_scenario, found, mocker):
    """
    Test contains protocol for Feature to check if it contains a Scenario
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add Scenarios to Feature
    for sentence in scenario_sentences:
        feature.scenarios.append(mocker.MagicMock(sentence=sentence))

    # when
    contains = expected_scenario in feature

    # then
    assert contains is found


@pytest.mark.parametrize('scenario_sentences, needle_scenario, expected_scenario', [
    (('foo', 'bar'), 'foo', 'foo'),
    (('bar'), 'foo', None),
    ([], 'foo', None)
])
def test_feature_get_scenario_as_item(scenario_sentences, needle_scenario, expected_scenario, mocker):
    """
    Test getitem protocol for Feature to get specific Scenario
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add Scenarios to Feature
    for sentence in scenario_sentences:
        feature.scenarios.append(mocker.MagicMock(sentence=sentence))

    # when
    actual_scenario = feature[needle_scenario]

    # then
    if expected_scenario is None:
        assert actual_scenario is None
    else:
        assert actual_scenario.sentence == expected_scenario


def test_feature_all_scenarios(mocker):
    """
    Test getting expanded list of all Scenarios of a Feature
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add regular Scenarios to Feature
    feature.scenarios.extend([mocker.MagicMock(id=1), mocker.MagicMock(id=2)])
    # add Scenario Outline to Feature
    feature.scenarios.append(
        mocker.MagicMock(spec=ScenarioOutline, id=3, scenarios=[
            mocker.MagicMock(id=4), mocker.MagicMock(id=5)]))
    # add Scenario Loop to Feature
    feature.scenarios.append(
        mocker.MagicMock(spec=ScenarioLoop, id=6, scenarios=[
            mocker.MagicMock(id=7), mocker.MagicMock(id=8)]))

    # when
    all_scenarios = feature.all_scenarios

    # then
    assert len(all_scenarios) == 8
    assert all_scenarios[0].id == 1
    assert all_scenarios[1].id == 2
    assert all_scenarios[2].id == 3
    assert all_scenarios[3].id == 4
    assert all_scenarios[4].id == 5
    assert all_scenarios[5].id == 6
    assert all_scenarios[6].id == 7
    assert all_scenarios[7].id == 8


def test_feature_constants(mocker):
    """
    Test getting all constants of this Feature
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add constants to Feature context -> this is directly done by the parser
    feature.context.constants = [mocker.MagicMock(value=1), mocker.MagicMock(value=2)]

    # when
    constants = feature.constants

    # then
    assert len(constants) == 2
    assert constants[0].value == 1
    assert constants[1].value == 2


def test_feature_state(mocker):
    """
    Test the state of a Feature according to the Scenario states
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add regular Scenarios to Feature
    regular_scenario = mocker.MagicMock(state=Step.State.PASSED)
    feature.scenarios.extend([regular_scenario, mocker.MagicMock(state=Step.State.PASSED)])
    # add Scenario Outline to Feature
    scenario_outline_example = mocker.MagicMock(state=Step.State.PASSED)
    scenario_outline = mocker.MagicMock(spec=ScenarioOutline, state=Step.State.PASSED, scenarios=[
        scenario_outline_example, mocker.MagicMock(state=Step.State.PASSED)
    ])
    feature.scenarios.append(scenario_outline)
    # add Scenario Loop to Feature
    scenario_loop_iteration = mocker.MagicMock(state=Step.State.PASSED)
    scenario_loop = mocker.MagicMock(spec=ScenarioLoop, state=Step.State.PASSED, scenarios=[
        scenario_loop_iteration, mocker.MagicMock(state=Step.State.PASSED)
    ])
    feature.scenarios.append(scenario_loop)

    # when all Scenarios pass then the Feature passes
    assert feature.state == Step.State.PASSED

    # when one Scenario is pending then the Feature is pending
    regular_scenario.state = Step.State.PENDING
    assert feature.state == Step.State.PENDING

    # when one Scenario is skipped then the Feature is skipped
    regular_scenario.state = Step.State.SKIPPED
    assert feature.state == Step.State.SKIPPED

    # when one Scenario is failed then the Feature is failed
    regular_scenario.state = Step.State.FAILED
    assert feature.state == Step.State.FAILED

    # when one Scenario is failed then the Feature is failed
    regular_scenario.state = Step.State.UNTESTED
    assert feature.state == Step.State.UNTESTED
    regular_scenario.state = Step.State.PASSED

    # Scenario Outline and Scenario Loop states are ignored
    scenario_outline.state = Step.State.FAILED
    assert feature.state == Step.State.PASSED
    scenario_loop.state = Step.State.FAILED
    assert feature.state == Step.State.PASSED

    # when a Scenario Outline Example is not passed the Feature is not passed
    scenario_outline_example.state = Step.State.FAILED
    assert feature.state == Step.State.FAILED
    scenario_outline_example.state = Step.State.PASSED

    # when a Scenario Loop Iteration is not passed the Feature is not passed
    scenario_loop_iteration.state = Step.State.FAILED
    assert feature.state == Step.State.FAILED
    scenario_loop_iteration.state = Step.State.PASSED

    # when a Scenario is untested which does not have to be run then the Feature is passed
    regular_scenario.state = Step.State.UNTESTED
    regular_scenario.has_to_run.return_value = False
    assert feature.state == Step.State.PASSED


@pytest.mark.parametrize('scenario_ids, scenario_choice, expected_has_to_run', [
    ((1, 2, 3), [], True),
    ((1, 2, 3), [1], True),
    ((1, 2, 3), [1, 2, 5], True),
    ((1, 2, 3), [4], False),
    ((1, 2, 3), [4, 5], False),
])
def test_feature_scenario_has_to_run(scenario_ids, scenario_choice, expected_has_to_run, mocker):
    """
    Test logic to check whether a Scenario within a Feature has to run or not
    """
    # given
    feature = Feature(1, 'Feature', 'I am a feature', 'foo.feature', 1, tags=None)
    # add Scenarios to Feature
    for scenario_id in scenario_ids:
        feature.scenarios.append(mocker.MagicMock(absolute_id=scenario_id))

    # when
    actual_has_to_run = feature.has_to_run(scenario_choice)

    # then
    assert actual_has_to_run is expected_has_to_run



# -*- coding: utf-8 -*-

from radish.stepregistry import step
from radish import given, when, then

@step("I have the number {number:g}")
def have_number(step, number):
    step.context.numbers.append(number)

@when("I sum them")
def sum_numbres(step):
    step.context.result = sum(step.context.numbers)

@then("I expect the result to be {result:g}")
def expect_result(step, result):
    assert step.context.result == result



# -*- coding: utf-8 -*-

"""
    This module provides a class to represent one Scenario Outline Example
"""

from .scenario import Scenario


class ExampleScenario(Scenario):
    """
        Represents one example scenario from a ScenarioOutline
    """
    def __init__(self, id, keyword, sentence, path, line, parent, example, background=None):
        super(ExampleScenario, self).__init__(id, keyword, sentence, path, line, parent, background=background)
        self.example = example

    def has_to_run(self, scenario_choice):
        """
            Returns wheiter the scenario has to run or not

            :param list scenario_choice: the scenarios to run. If None all will run
        """
        return self.parent.has_to_run(scenario_choice)



# -*- coding: utf-8 -*-

import re

from radish.stepregistry import step
from radish import given, when, then

@step("I have the number {number:g}")
def have_number(step, number):
    step.context.numbers.append(number)


@step(re.compile("I have the numbers (.*)"))
def have_number(step, numbers):
    numbers = [x.strip() for x in numbers.split(',')]

    for n in numbers:
        step.behave_like('I have the number {0}'.format(n))


@when("I sum them")
def sum_numbers(step):
    step.context.result = sum(step.context.numbers)


@then("I expect the result to be {result:g}")
def expect_result(step, result):
    assert step.context.result == result



# -*- coding: utf-8 -*-

from radish import before

@before.each_scenario
def init_numbers(scenario):
    scenario.context.users = []
    scenario.context.database = lambda: None
    setattr(scenario.context.database, "users", [])
    scenario.context.database.users = []



# -*- coding: utf-8 -*-

"""
    This module provides a registry for all custom steps which were decorated with the @step-decorator.
"""

import re
import inspect
from singleton import singleton

from .exceptions import RadishError, SameStepError, StepRegexError


@singleton()
class StepRegistry(object):
    """
        Represents the step registry
    """
    def __init__(self):
        self._steps = {}

    def register(self, pattern, func):
        """
            Registers a given regex with the given step function.
        """
        if pattern in self._steps:
            raise SameStepError(pattern, self._steps[pattern], func)

        self._steps[pattern] = func

    def get_pattern(self, func):
        """
            Get step pattern from a given function.
        """
        return next((k for k, v in self._steps.items() if v == func), "Unknown")

    def register_object(self, steps_object):
        """
            Registers all public methods from the given object as steps.
            The step regex must be in the first line of the docstring

            :param instance steps_object: the object with step definition methods
        """
        ignore = getattr(steps_object, "ignore", [])
        for attr in dir(steps_object):
            if attr in ignore or not inspect.ismethod(getattr(steps_object, attr)):  # attribute should be ignored
                continue

            step_definition = getattr(steps_object, attr)
            step_regex = self._extract_regex(step_definition)
            self.register(step_regex, step_definition)

        return steps_object

    @staticmethod
    def _extract_regex(func):
        """
            Extracts a step regex from the docstring of the given func

            :param function func: the step definition function
        """
        docstr = func.__doc__.strip() if func.__doc__ else None
        if not docstr:
            raise RadishError("Step definition '{0}' from class must have step regex in docstring".format(func.__name__))

        regex = docstr.splitlines()[0]
        try:
            re.compile(regex)
        except re.error as e:
            raise StepRegexError(regex, func.__name__, e)

        return regex

    def clear(self):
        """
            Clears all registered steps
        """
        self._steps = {}

    @property
    def steps(self):
        """
            Returns all registered steps
        """
        return self._steps


def step(pattern):
    """
        Step decorator for custom steps

        :param string regex: this is the regex to match the steps in the feature file

        :returns: the decorated function
        :rtype: function
    """
    def _decorator(func):
        """
            Represents the actual decorator
        """
        StepRegistry().register(pattern, func)
        return func
    return _decorator


def steps(cls):
    """
        Decorator for classes with step definitions inside
    """
    old_cls_init = getattr(cls, "__init__")

    # ignore new_cls_init method
    if not hasattr(cls, 'ignore'):
        cls.ignore = []

    cls.ignore.append('__init__')

    def new_cls_init(self, *args, **kwargs):
        """
            New __init__ method for the given class which calls the old __init__ method
            and registers the steps
        """
        old_cls_init(self, *args, **kwargs)

        # register functions as step
        StepRegistry().register_object(self)

    setattr(cls, "__init__", new_cls_init)
    # create instance in order to register object in step registry
    cls()
    return cls


def given(pattern):
    """
        Step decorator prefixed with the Given-keyword.
    """
    if isinstance(pattern, re._pattern_type):  # pylint: disable=protected-access
        return step(re.compile(r"Given {0}".format(pattern.pattern)))
    return step("Given {0}".format(pattern))


def when(pattern):
    """
        Step decorator prefixed with the When-keyword.
    """
    if isinstance(pattern, re._pattern_type):  # pylint: disable=protected-access
        return step(re.compile(r"When {0}".format(pattern.pattern)))
    return step("When {0}".format(pattern))


def then(pattern):
    """
        Step decorator prefixed with the Then-keyword.
    """
    if isinstance(pattern, re._pattern_type):  # pylint: disable=protected-access
        return step(re.compile(r"Then {0}".format(pattern.pattern)))
    return step("Then {0}".format(pattern))



# -*- coding: utf-8 -*-

"""
    This module provides an extension which starts a python shell after a step failed
"""

from radish.hookregistry import after
from radish.stepmodel import Step
from radish.exceptions import RadishError
from radish.extensionregistry import extension


@extension
class FailureInspector(object):
    """
        Failure inspector radish extension
    """
    OPTIONS = [("--inspect-after-failure", "start python shell after failure")]
    LOAD_IF = staticmethod(lambda config: config.inspect_after_failure)
    LOAD_PRIORITY = 10

    def __init__(self):
        after.each_step(self.failure_inspector)

    def failure_inspector(self, step):
        """
            Starts a python shell after a step failed
        """
        if step.state is not Step.State.FAILED:
            return

        try:
            from IPython import embed
        except ImportError as e:
            raise RadishError('if you want to use the failure inspector extension you have to "pip install radish-bdd[ipython-debugger]"')

        embed()



# -*- coding: utf-8 -*-

"""
    Provide plugin interface for radish extensions
"""

from singleton import singleton


@singleton()
class ExtensionRegistry(object):
    """
        Registers all extensions
    """
    DEFAULT_LOAD_PRIORITY = 1000

    def __init__(self):
        self.extensions = []
        self.loaded_extensions = []

    def reset(self):
        """
        Reset all registered extensions
        """
        self.extensions = []
        self.loaded_extensions = []

    def register(self, extension_class):
        """
            Registers the class as a radish extension
        """
        self.extensions.append(extension_class)

    def load(self, config):
        """
            Loads all needed extensions
        """
        for ext in sorted(self.extensions, key=lambda x: getattr(x, "LOAD_PRIORITY", self.DEFAULT_LOAD_PRIORITY)):
            try:
                if ext.LOAD_IF(config):
                    self.loaded_extensions.append(ext())
            except AttributeError:
                pass

    def get_options(self):
        """
            Returns all options registered by plugins
        """
        options = []
        for ext in self.extensions:
            try:
                options.extend(opt[0] for opt in ext.OPTIONS)
            except AttributeError:
                pass
        return "\n           ".join("[{0}]".format(x) for x in options)

    def get_option_description(self):
        """
            Returns all option descriptions registerd by plugins
        """
        options = []
        for ext in self.extensions:
            try:
                options.extend("{0} {1}".format(opt[0].ljust(43), opt[1]) for opt in ext.OPTIONS)
            except AttributeError:
                pass
        return "\n    ".join(options)


def extension(klass):
    """
        Registers the class as a radish extension
    """
    ExtensionRegistry().register(klass)
    return klass



# -*- coding: utf-8 -*-

import re
from radish.terrain import world
from radish.stepregistry import step

world.number = 0

@step("I have the number {number:g}")
def have_number(step, number):
    world.number = int(number)


@step(re.compile(r"I add (\d+) to my number"))
def add_to_number(step, addition):
    world.number += int(addition)
    if int(addition) == 3:
        world.number += 1
    if int(addition) == 11:
        assert False, "ANTOHER ERROR °"


@step("I expect the number to be {:MathExpression}")
def expect_number(step, number):
    assert world.number == int(number), "Expected number to be {0}. Actual number is: {1}".format(number, world.number)


@step("I prepare users in world object")
def prepare_users(step):
    world.users = None
    #assert False, "Some bug"


@step("I initialize users in world object")
def init_users(step):
    world.users = []


@step("I setup the database")
def setup_database(step):
    step.behave_like("I prepare users in world object")
    step.behave_like("I initialize users in world object")


@step("I add the users")
def add_user(step):
    for row in step.table:
        world.users.append({"forename": row[0], "surname": row[1], "email": row[2]})


@step(re.compile(r"I expect the user \"(.*?)\" in the database"))
def expect_user_in_database(step, name):
    forename, surname = name.split()
    assert any(u for u in world.users if u["forename"] == forename and u["surname"] == surname), "No such user in the db"


@step("I have the numeric expression {expression:MathExpression}")
def have_expression(step, expression):
    world.result = expression


@step("I add {addition:MathExpression} to this")
def add_to_result(step, addition):
    world.result += addition


@step("I expect the result to be {result:MathExpression}")
def expect_result(step, result):
    assert world.result == result, "Result is {0} but expected {1}".format(world.result, result)


@step("I have the float number {:g}")
def have_float_number(step, number):
    world.float_number = number


@step("I add to my float number {:g}")
def add_to_float_number(step, number):
    world.float_number += number


@step("I expect the float result to be {result:g}")
def expect_float_number(step, result):
    assert world.float_number == result, "Result is {0} but expected {1}".format(world.float_number, result)


@step("I have the following data:")
def have_data(step):
    step.context.some_data = step.text


@step("I ignore this step")
def ignore_step(step):
    pass


@step("I expect the data to be \"{}\"")
def expect_data(step, expected_data):
    assert step.context.some_data == expected_data, "Data is: '{0}'. Expected was: '{1}'".format(step.context.some_data, expected_data)

@step("Given I install the database server")
def demo(step):
    pass


@step("When I add all default users")
def demo(step):
    pass


@step("And I add all default types")
def demo(step):
    pass


@step("And I initialize the database error handlers")
def demo(step):
    pass


@step("And I set the permissions for the administrator tables")
def demo(step):
    pass


@step("And I index imported fields")
def demo(step):
    pass


@step("Then I expect my installation to be complete")
def demo(step):
    pass

@step("I add the user \"Timo furrer\"")
def demo(step):
    pass


@step("I expect the user \"Timo Furrer\" in the databas")
def demo(step):
    pass



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import pytest

from radish.runner import Runner
from radish.stepmodel import Step


@pytest.mark.parametrize('run_state, expected_returncode', [
    (Step.State.PASSED, 0),
    (Step.State.UNTESTED, 0),
    (Step.State.SKIPPED, 0),
    (Step.State.PENDING, 0),
    (Step.State.FAILED, 1),
], ids=[
    'Running Step returing PASSED',
    'Running Step returing UNTESTED',
    'Running Step returing SKIPPED',
    'Running Step returing PENDING',
    'Running Step returing FAILED',
])
def test_run_single_step(run_state, expected_returncode, hookregistry, mocker):
    """
    Test running a single Step
    """
    # given
    runner = Runner(hookregistry)
    step = mocker.MagicMock()
    step.run.return_value = run_state

    # when
    returncode = runner.run_step(step)

    # then
    assert returncode == expected_returncode
    assert step.run.call_count == 1


@pytest.mark.parametrize('debug_state, expected_returncode', [
    (Step.State.PASSED, 0),
    (Step.State.UNTESTED, 0),
    (Step.State.SKIPPED, 0),
    (Step.State.PENDING, 0),
    (Step.State.FAILED, 1),
], ids=[
    'Debugging Step returing PASSED',
    'Debugging Step returing UNTESTED',
    'Debugging Step returing SKIPPED',
    'Debugging Step returing PENDING',
    'Debugging Step returing FAILED',
])
def test_debug_single_step(debug_state, expected_returncode, world_config, hookregistry, mocker):
    """
    Test debugging a single Step
    """
    # given
    runner = Runner(hookregistry)
    step = mocker.MagicMock()
    step.debug.return_value = debug_state

    # set debug mode
    world_config.debug_steps = True

    # when
    returncode = runner.run_step(step)

    # then
    assert returncode == expected_returncode
    assert step.debug.call_count == 1


def test_run_single_step_show_only(hookregistry, mocker):
    """
    Test running single step when show only mode is on
    """
    # given
    runner = Runner(hookregistry, show_only=True)
    step = mocker.MagicMock()
    step.run.return_value = Step.State.FAILED

    # when
    returncode = runner.run_step(step)

    # then
    assert returncode == 0
    assert step.run.call_count == 0


def test_skip_single_step(hookregistry, mocker):
    """
    Test skipping a single Step
    """
    # given
    runner = Runner(hookregistry)
    step = mocker.MagicMock()
    step.skip.return_value = None

    # when
    runner.skip_step(step)

    # then
    assert step.skip.call_count == 1


@pytest.mark.parametrize('run_or_skip', [
    'run_step', 'skip_step'
])
def test_run_skip_step_hooks(run_or_skip, hookregistry, mocker):
    """
    Test that Hooks are executed when running or skipping a Step
    """
    # given
    # register hooks in registry
    before_step_stub = mocker.stub()
    after_step_stub = mocker.stub()
    hookregistry.register('before', 'each_step', before_step_stub)
    hookregistry.register('after', 'each_step', after_step_stub)
    # create runner
    runner = Runner(hookregistry, show_only=True)

    step = mocker.MagicMock(all_tags=[])

    # when
    method = getattr(runner, run_or_skip)
    method(step)

    # then
    assert before_step_stub.call_count == 1
    assert after_step_stub.call_count == 1



# -*- coding: utf-8 -*-

"""
This module provide Python 2 / Python 3 compatability
"""

import sys

try:
    from io import StringIO
except ImportError:
    from StringIO import StringIO


# flags to indicate Python versions
PY2 = sys.version_info[0] == 2

# RecursionError does not exist in Python < 3.5
RecursionError = RuntimeError if sys.version_info < (3, 5) else RecursionError


def u(text):  # pragma: no cover
    """
    Encode given text to unicode utf-8 in manner that works accross various
    python interpreters and version. Currently only support CPython 2 and 3.

    :param text: text to encode
    :type text: str,unicode
    """
    # look for unicode in the builtins which only exists in python 2
    if PY2 is True:
        return unicode(text)

    return str(text)



# -*- coding: utf-8 -*-

"""
    This model provides a base class for all models:

    Models:
        * Feature
        * Scenario
        * ScenarioOutline
        * Step
"""

from .exceptions import RadishError


class Tag(object):
    """
    Represents a tag for a model
    """
    def __init__(self, name, arg=None):
        self.name = name
        self.arg = arg

    def __hash__(self):
        return hash(self.name, self.arg)

    def __eq__(self, other):
        if not isinstance(other, Tag):
            return False

        return self.name == other.name and self.arg == other.arg


# FIXME: make ABC
class Model(object):
    """
        Represents a base model
    """
    class Context(object):
        """
            Represents a Models context.
            For every feature/scenario a new Context object is created
        """
        def __init__(self):
            self.constants = []

    def __init__(self, id, keyword, sentence, path, line, parent=None, tags=None):
        self.id = id
        self.keyword = keyword
        self.sentence = sentence
        self.path = path
        self.line = line
        self.parent = parent
        self.tags = tags or []
        self.starttime = None
        self.endtime = None

    @property
    def all_tags(self):
        """
        Return all tags for this model and all it's parents
        """
        tags = []
        if self.parent:
            tags.extend(self.parent.all_tags)
        return tags + self.tags

    @property
    def duration(self):
        """
            Returns the duration of this model
        """
        if not self.starttime or not self.endtime:
            raise RadishError("Cannot get duration of {0} '{1}' because either starttime or endtime is not set".format(self.keyword, self.sentence))

        return self.endtime - self.starttime



# -*- coding: utf-8 -*-

"""
    This module provides a class to match the feature file steps with the registered steps from the registry
"""

import re
from collections import namedtuple

from parse_type.cfparse import Parser

from .customtyperegistry import CustomTypeRegistry
from .exceptions import StepDefinitionNotFoundError, StepPatternError

StepMatch = namedtuple("StepMatch", ["argument_match", "func"])


class RegexStepArguments(object):  # pylint: disable=too-few-public-methods
    """Class to represent the argument groups matched by a regex step pattern"""

    def __init__(self, match):
        self.match = match

    def evaluate(self):
        """Lazy and return evaluate the step group matches"""
        return self.match.groups(), self.match.groupdict()


class ParseStepArguments(object):  # pylint: disable=too-few-public-methods
    """Class to represent the argument groups matched by a parse step pattern"""

    def __init__(self, match):
        self.match = match

    def evaluate(self):
        """Lazy and return evaluate the step group matches"""
        result = self.match.evaluate_result()
        return result.fixed, result.named


def merge_steps(features, steps):
    """
        Merges steps from the given features with the given steps

        :param list features: the features
        :param dict steps: the steps
    """
    # FIXME: fix cycle-import ... Matcher -> ScenarioOutline -> Step -> Matcher
    from .scenariooutline import ScenarioOutline
    for feature in features:
        for scenario in feature.all_scenarios:
            if scenario.background:
                for step in scenario.background.steps:
                    merge_step(step, steps)

            if isinstance(scenario, ScenarioOutline):
                continue  # ScenarioOutline steps do not have to be merged

            for step in scenario.steps:
                merge_step(step, steps)


def merge_step(step, steps):
    """
        Merges a single step with the registered steps

        :param Step step: the step from a feature file to merge
        :param list steps: the registered steps
    """
    match = match_step(step.context_sensitive_sentence, steps)
    if not match or not match.func:
        raise StepDefinitionNotFoundError(step)

    step.definition_func = match.func
    step.argument_match = match.argument_match


def match_step(sentence, steps):
    """
        Tries to find a match from the given sentence with the given steps

        :param string sentence: the step sentence to match
        :param dict steps: the available registered steps

        :returns: the arguments and the func which were matched
        :rtype: tuple
    """
    potentional_matches = []
    for pattern, func in steps.items():
        if isinstance(pattern, re._pattern_type):  # pylint: disable=protected-access
            match = pattern.search(sentence)
            argument_match = RegexStepArguments(match)
            if match:
                longest_group = get_longest_group(match)
            else:
                longest_group = 0
        else:
            try:
                parser = Parser(pattern, CustomTypeRegistry().custom_types)
            except ValueError as e:
                raise StepPatternError(pattern, func.__name__, e)

            match = parser.search(sentence, evaluate_result=False)
            argument_match = ParseStepArguments(match)
            if match:
                longest_group = get_longest_group(match.match)
            else:
                longest_group = 0

        if match:
            step_match = StepMatch(argument_match=argument_match, func=func)
            if len(sentence) == longest_group:
                # if perfect match can be made we return it no
                # matter of the other potentional matches
                return step_match

            distance_to_perfect = abs(len(sentence) - longest_group)
            potentional_matches.append((step_match, distance_to_perfect))

    if potentional_matches:
        # get best match
        return min(potentional_matches, key=lambda x: x[1])[0]

    return None


def get_longest_group(match):
    # FIXME(TF): check if match.start() and match.end() matches sentence?!
    return len(match.group())



# -*- coding: utf-8 -*-

__DESCRIPTION__ = "Behaviour-Driven-Development tool for python"
__LICENSE__ = "MIT"
__VERSION__ = "0.8.5"
__AUTHOR__ = "Timo Furrer"
__AUTHOR_EMAIL__ = "tuxtimo@gmail.com"
__URL__ = "http://radish-bdd.io"
__DOWNLOAD_URL__ = "https://github.com/radish-bdd/radish"
__BUGTRACK_URL__ = "https://github.com/radish-bdd/radish/issues"

# export some functions for users
from .terrain import world, pick
from .hookregistry import before, after
from .stepregistry import step, given, when, then, steps
from .customtyperegistry import custom_type, register_custom_type, TypeBuilder
from .extensionregistry import extension
from .exceptions import ValidationError



# -*- coding: utf-8 -*-


class Hero(object):
    def __init__(self, forename, surname, hero):
        self.forename = forename
        self.surname = surname
        self.heroname = hero

    def __repr__(self):
        return 'Hero({0}, {1}, {2})'.format(
            self.forename, self.surname, self.heroname)



# -*- coding: utf-8 -*-

"""
    This module contains a class to load the step and terrain files
"""

import os
import sys
import fnmatch

from . import utils


def load_modules(location):
    """
        Loads all modules in the `location` folder
    """
    location = os.path.expanduser(os.path.expandvars(location))
    if not os.path.exists(location):
        raise OSError("Location '{0}' to load modules does not exist".format(location))

    for p, _, f in os.walk(location):
        for filename in fnmatch.filter(f, "*.py"):
            load_module(os.path.join(p, filename))


def load_module(path):
    """
        Loads a module by the given `path`

        :param string path: the path to the module to load
    """
    module_name = os.path.splitext(os.path.split(path)[1])[0]
    try:
        if sys.version_info >= (3, 5):
            # the imp module is deprecated since Python 3.6
            import importlib.util
            spec = importlib.util.spec_from_file_location(module_name, path)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
        else:
            import imp
            parent = os.path.dirname(utils.expandpath(path))
            f, pathname, desc = imp.find_module(module_name, [parent])
            imp.load_module(module_name, f, pathname, desc)
            f.close()
    except Exception as e:
        raise ImportError("Unable to import module '{0}' from '{1}': {2}".format(
              module_name, path, e))



# -*- coding: utf-8 -*-

"""
    radish
    ~~~~~~

    Behavior Driven Development tool for Python - the root from red to green

    Copyright: MIT, Timo Furrer <tuxtimo@gmail.com>
"""

import re

import pytest

from parse_type.cfparse import Parser

import radish.matcher as matcher
import radish.exceptions as errors


@pytest.mark.parametrize('regex, string, expected_args, expected_kwargs', [
    (r'Given I have the number (\d+)', 'Given I have the number 5', ('5',), {}),
    (r'Given I have the number (?P<number>\d+)', 'Given I have the number 5', ('5',), {'number': '5'}),
    (r'Given I have the number (?P<number>\d+) and (\d+)', 'Given I have the number 4 and 2', ('4', '2'), {'number': '4'}),
], ids=[
    'Regex with unnamed groups',
    'Regex with named groups',
    'Regex with unnamed and named groups'
])
def test_regex_step_arguments_object(regex, string, expected_args, expected_kwargs):
    """
    Test functionality of RegexStepArguments object
    """
    # given
    match = re.search(regex, string)
    args = matcher.RegexStepArguments(match)

    # when
    actual_args, actual_kwargs = args.evaluate()

    # then
    assert actual_args == expected_args
    assert actual_kwargs == expected_kwargs


@pytest.mark.parametrize('parse_pattern, string, expected_args, expected_kwargs', [
    ('Given I have the number {:d}', 'Given I have the number 5', (5,), {}),
    ('Given I have the number {number:d}', 'Given I have the number 5', tuple(), {'number': 5}),
    ('Given I have the number {number:d} and {:d}', 'Given I have the number 4 and 2', (2,), {'number': 4}),
], ids=[
    'Parse Pattern with unnamed groups',
    'Parse Pattern with named groups',
    'Parse Pattern with unnamed and named groups'
])
def test_parse_step_arguments_object(parse_pattern, string, expected_args, expected_kwargs):
    """
    Test functionality of ParseStepArguments object
    """
    # given
    parser = Parser(parse_pattern)
    match = parser.search(string, evaluate_result=False)
    args = matcher.ParseStepArguments(match)

    # when
    actual_args, actual_kwargs = args.evaluate()

    # then
    assert actual_args == expected_args
    assert actual_kwargs == expected_kwargs


@pytest.mark.parametrize('given_sentence, given_steps, expected_argument_match_type, expected_func_match', [
    (
        'Given I have a number', {re.compile('Given I have a number'): 1}, matcher.RegexStepArguments, 1
    ),
    (
        'Given I have a number to test', {re.compile('Given I have a number'): 1}, matcher.RegexStepArguments, 1
    ),
    (
        'Given I have a number', {
            re.compile('Given I have some number'): 1,
            re.compile('Given I have a number'): 2
        }, matcher.RegexStepArguments, 2
    ),
    (
        'Given I have a number', {
            re.compile('I have a number'): 1,
            re.compile('Given I have a number'): 2
        }, matcher.RegexStepArguments, 2
    ),
    (
        'Given I have the number 5', {
            re.compile(r'I have the number (\d+)'): 1,
            re.compile(r'Given I have the number (\d+)'): 2
        }, matcher.RegexStepArguments, 2
    ),
    (
        'Given I have the number 5 which I use to test', {
            re.compile(r'Given I have the number (\d+) which I use to do some stuff'): 1,
            re.compile(r'Given I have the number (\d+) which I use'): 2
        }, matcher.RegexStepArguments, 2
    ),
    # Parse Argument Matches
    (
        'Given I have a number', {'Given I have a number': 1}, matcher.ParseStepArguments, 1
    ),
    (
        'Given I have a number to test', {'Given I have a number': 1}, matcher.ParseStepArguments, 1
    ),
    (
        'Given I have a number', {
            'Given I have some number': 1,
            'Given I have a number': 2
        }, matcher.ParseStepArguments, 2
    ),
    (
        'Given I have a number', {
            'I have a number': 1,
            'Given I have a number': 2
        }, matcher.ParseStepArguments, 2
    ),
    (
        'Given I have the number 5', {
            'I have the number {:d}': 1,
            'Given I have the number {:d}': 2
        }, matcher.ParseStepArguments, 2
    ),
    (
        'Given I have the number 5 which I use to test', {
            'Given I have the number {:d} which I use to do some stuff': 1,
            'Given I have the number {:d} which I use': 2
        }, matcher.ParseStepArguments, 2
    ),
], ids=[
    'Regex: match simple step with single candidate',
    'Regex: match simple step with single candidate which is not a perfect match',
    'Regex: match simple step with one matching candidate',
    'Regex: match simple step with multiple candidate - best match',
    'Regex: match step with args with multiple candidate - best match',
    'Regex: match step with args with multiple candidate - closest match',
    # Parse Argument Matches
    'Parse: match simple step with single candidate',
    'Parse: match simple step with single candidate which is not a perfect match',
    'Parse: match simple step with one matching candidate',
    'Parse: match simple step with multiple candidate - best match',
    'Parse: match step with args with multiple candidate - best match',
    'Parse: match step with args with multiple candidate - closest match',
])
def test_match_sentence_with_steps(given_sentence, given_steps, expected_argument_match_type,
                                   expected_func_match):
    """
    Test matching a sentence with given Steps
    """
    # given & when
    match = matcher.match_step(given_sentence, given_steps)

    # then
    assert isinstance(match.argument_match, expected_argument_match_type)
    assert match.func == expected_func_match


@pytest.mark.parametrize('given_sentence, given_steps', [
    ('Given I have the number', {re.compile('Given I have a number'): 1}),
    ('Given I have the number foo', {re.compile(r'Given I have number (\d+)'): 1}),
    ('Given I have the number', {'Given I have a number': 1}),
    ('Given I have the number foo', {'Given I have number {:d}': 1})
], ids=[
    'Regex: simple not matching step',
    'Regex: not matching step with arguments',
    'Parse: simple not matching step',
    'Parse: not matching step with arguments'
])
def test_no_step_match(given_sentence, given_steps):
    """
    Test failing to match a sentence with given Steps
    """
    # given & when
    match = matcher.match_step(given_sentence, given_steps)

    # then
    assert match is None


def test_invalid_parse_pattern():
    """
    Test failure for invalid Parse pattern
    """
    # given
    invalid_pattern = 'Given I have the number {:d {}'

    # when
    with pytest.raises(errors.StepPatternError) as exc:
        matcher.match_step('Given I have the number 5', {invalid_pattern: int})

    assert str(exc.value).startswith("Cannot compile pattern 'Given I have the number {:d {}' of step 'int': ")


@pytest.mark.parametrize('given_sentence, given_steps, expected_func, expected_args, expected_kwargs', [
    ('Given I have a number', {re.compile(r'Given I have a number'): 1}, 1, tuple(), {}),
    ('Given I have the number 4 and 2', {
        re.compile(r'Given I have the number (?P<number>\d+) and (\d+)'): 1}, 1,
     ('4', '2'), {'number': '4'}),
    ('Given I have a number', {'Given I have a number': 1}, 1, tuple(), {}),
    ('Given I have the number 4 and 2', {
        'Given I have the number {number:d} and {:d}': 1}, 1,
     (2,), {'number': 4})
], ids=[
    'Regex: simple step merge',
    'Regex: step merge with arguments',
    'Parse: simple step merge',
    'Parse: step merge with arguments',
])
def test_merging_step(given_sentence, given_steps, expected_func, expected_args, expected_kwargs, mocker):
    """
    Test merging a Step with registered Step functions
    """
    # given
    step_to_merge = mocker.MagicMock(
        context_sensitive_sentence=given_sentence,
        definition_func=None, argument_match=None)

    # when
    matcher.merge_step(step_to_merge, given_steps)
    actual_args, actual_kwargs = step_to_merge.argument_match.evaluate()

    # then
    assert step_to_merge.definition_func == expected_func
    assert actual_args == expected_args
    assert actual_kwargs == expected_kwargs


@pytest.mark.parametrize('given_sentence, given_steps', [
    ('Given I have the number', {re.compile('Given I have a number'): 1}),
    ('Given I have the number foo', {re.compile(r'Given I have number (\d+)'): 1}),
    ('Given I have the number', {'Given I have a number': 1}),
    ('Given I have the number foo', {'Given I have number {:d}': 1})
], ids=[
    'Regex: simple not matching step',
    'Regex: not matching step with arguments',
    'Parse: simple not matching step',
    'Parse: not matching step with arguments'
])
def test_no_step_definition_merge(given_sentence, given_steps, mocker):
    """
    Test failure when no Step definition function can be found during Step merge
    """
    # given
    step_to_merge = mocker.MagicMock(
        context_sensitive_sentence=given_sentence,
        definition_func=None, argument_match=None)

    # when
    with pytest.raises(errors.StepDefinitionNotFoundError) as exc:
        matcher.merge_step(step_to_merge, given_steps)

    # then
    assert str(exc.value).startswith('Cannot find step definition for step')



# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring

import os
import re
import codecs
from setuptools import setup, find_packages


def read_metafile(path):
    """
    Read contents from given metafile
    """
    with codecs.open(path, 'rb', 'utf-8') as f:
        return f.read()


def get_meta(name):
    """
    Get some metdata with the give name from the
    Meta file
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=name.upper()),
        __META_DATA__, re.M
    )

    if not meta_match:
        raise RuntimeError('Unable to find __{0}__ string.'.format(name))
    return meta_match.group(1)


__META_FILE__ = os.path.join('radish', '__init__.py')
__META_DATA__ = read_metafile(__META_FILE__)
__README_CONTENTS__ = read_metafile('README.md')


# mandatory requirements for the radish base features
requirements = [
    'docopt',
    'pysingleton',
    'colorful>=0.3.11',
    'tag-expressions>=1.0.0',
    # FIXME(TF): this is a forked repository with
    #            the parse version included we need.
    #            change to upstream parse_type asap.
    'radish-parse_type>0.3.4',
    'humanize'
]


# optional requirements used by extensions which are
# disabled by default
extra_requirements = {
    'bddxml': ['lxml'],
    'ipython-debugger': ['ipython'],
    'coverage': ['coverage'],
    'testing': ['PyYAML']
}

setup(
    name='radish-bdd',
    version=get_meta('version'),
    license=get_meta('license'),
    description=get_meta('description'),
    long_description=__README_CONTENTS__,
    long_description_content_type='text/markdown',
    author=get_meta('author'),
    author_email=get_meta('author_email'),
    maintainer=get_meta('author'),
    maintainer_email=get_meta('author_email'),
    platforms=['Linux', 'Windows', 'MAC OS X'],
    url=get_meta('url'),
    download_url=get_meta('download_url'),
    bugtrack_url=get_meta('bugtrack_url'),
    packages=find_packages(),
    package_data={'': ['radish/languages/*', '*.md']},
    install_requires=requirements,
    extras_require=extra_requirements,
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'radish = radish.main:main',
            'radish-test = radish.testing.__main__:main [testing]',
        ]},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: OS Independent',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: Microsoft :: Windows :: Windows XP',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation',
        'Topic :: Education :: Testing',
        'Topic :: Software Development',
        'Topic :: Software Development :: Testing'
    ],
)



# -*- coding: utf-8 -*-

"""
This module provides functionaliy to test
if some sentences are matched with the expected
step implementations.
"""

import sys
import codecs

import yaml
import colorful

from radish.loader import load_modules
from radish.matcher import match_step
from radish.stepregistry import StepRegistry
from radish.utils import get_func_arg_names, get_func_location, locate
from radish.compat import u


def test_step_matches_configs(match_config_files, basedirs, cover_min_percentage=None, cover_show_missing=False):
    """
    Test if the given match config files matches the actual
    matched step implementations.
    """
    if cover_min_percentage is not None and float(cover_min_percentage) > 100:
        sys.stderr.write(str(colorful.magenta('You are a little cocky to think you can reach a minimum coverage of {0:.2f}%\n'.format(
            float(cover_min_percentage)))))
        return 3

    # load user's custom python files
    for basedir in basedirs:
        load_modules(basedir)

    steps = StepRegistry().steps

    if not steps:
        sys.stderr.write(str(colorful.magenta('No step implementations found in {0}, thus doesn\'t make sense to continue'.format(
            basedirs))))
        return 4

    failed = 0
    passed = 0
    covered_steps = set()

    for match_config_file in match_config_files:
        # load the given match config file
        with codecs.open(match_config_file, "r", "utf-8") as f:
            match_config = yaml.safe_load(f)

        if not match_config:
            print(colorful.magenta('No sentences found in {0} to test against'.format(match_config_file)))
            return 5

        print(colorful.yellow('Testing sentences from {0}:'.format(colorful.bold_yellow(match_config_file))))
        failed_sentences, passed_senteces = test_step_matches(match_config, steps)
        failed += failed_sentences
        passed += passed_senteces

        covered_steps = covered_steps.union(x['should_match'] for x in match_config if 'should_match' in x)

        # newline
        sys.stdout.write('\n')

    report = colorful.bold_white('{0} sentences ('.format(failed + passed))
    if passed > 0:
        report += colorful.bold_green('{0} passed'.format(passed))

    if passed > 0 and failed > 0:
        report += colorful.bold_white(', ')

    if failed > 0:
        report += colorful.bold_red('{0} failed'.format(failed))
    report += colorful.bold_white(')')
    print(report)

    step_coverage = 100.0 / len(steps) * len(covered_steps)
    coverage_report = colorful.bold_white('Covered {0} of {1} step implementations'.format(
        len(covered_steps), len(steps)))

    ret = 0 if failed == 0 else 1

    if cover_min_percentage:
        coverage_color = colorful.bold_green if step_coverage >= float(cover_min_percentage) else colorful.bold_red
        coverage_report += colorful.bold_white(' (coverage: ')
        coverage_report += coverage_color('{0:.2f}%'.format(step_coverage))
        if float(cover_min_percentage) > step_coverage:
            coverage_report += colorful.bold_white(', expected a minimum of {0}'.format(
                colorful.bold_green(cover_min_percentage + '%')))
            if failed == 0:
                ret = 2
            # if tests have passed and coverage is too low we fail with exit code 2
        coverage_report += colorful.bold_white(')')

    print(coverage_report)

    if cover_show_missing:
        missing_steps = get_missing_steps(steps, covered_steps)
        if missing_steps:
            missing_step_report = colorful.bold_yellow('Missing steps:\n')
            for step in missing_steps:
                missing_step_report += '- {0} at '.format(
                    colorful.cyan(step[0]))
                missing_step_report += colorful.cyan(step[1]) + '\n'
            sys.stdout.write(missing_step_report)

    return ret


def test_step_matches(match_config, steps):
    """
    Test if the given match config matches the actual
    matched step implementations.
    """
    failed = 0
    passed = 0

    for item in match_config:
        validate_config_item(item)

        sentence = item['sentence']

        if 'should_match' in item:
            has_passed = test_step_match(sentence, item['should_match'],
                                         item.get('with_arguments', None), steps)
        else:
            has_passed = test_step_not_match(sentence, item['should_not_match'], steps)

        if has_passed:
            passed += 1
        else:
            failed += 1

    return failed, passed


def test_step_match(sentence, expected_step, expected_arguments, steps):
    sys.stdout.write('{0} STEP "{1}" SHOULD MATCH {2}    '.format(
        colorful.yellow('>>'), colorful.cyan(sentence), colorful.cyan(expected_step)))

    result = match_step(sentence, steps)
    if not result:
        output_failure(None, ['Expected sentence didn\'t match any step implementation'])
        return False

    if expected_step != result.func.__name__:
        output_failure(result.func,
                       ['Expected sentence matched {0} instead of {1}'.format(
                           result.func.__name__, expected_step)])
        return False

    if expected_arguments:
        arguments = merge_step_args(result)
        expected_arguments = {k: v for expected_arguments in expected_arguments
                              for k, v in expected_arguments.items()}
        argument_errors = check_step_arguments(expected_arguments, arguments)
        if argument_errors:
            output_failure(result.func, argument_errors)
            return False

    print(u(colorful.bold_green(u'✔')))
    return True


def test_step_not_match(sentence, expected_not_matching_step, steps):
    step_to_print = colorful.cyan(expected_not_matching_step) if expected_not_matching_step else 'ANY'
    sys.stdout.write('{0} STEP "{1}" SHOULD NOT MATCH {2}    '.format(
        colorful.yellow('>>'), colorful.cyan(sentence), step_to_print))

    result = match_step(sentence, steps)
    if result:
        if not expected_not_matching_step or result.func.__name__ == expected_not_matching_step:
            output_failure(None, ['Expected sentence did match {0} but it shouldn\'t'.format(
                expected_not_matching_step)])
            return False

    print(u(colorful.bold_green(u'✔')))
    return True


VALID_CONFIG_ITEMS = {'sentence', 'should_match', 'should_not_match', 'with_arguments'}


def validate_config_item(config):
    """
    Validate the given config object
    """
    given_attributes = set(config.keys())
    if not given_attributes.issubset(VALID_CONFIG_ITEMS):
        raise ValueError('The config attributes {0} are invalid. Use only {1}'.format(
            ', '.join(sorted(given_attributes.difference(VALID_CONFIG_ITEMS))), ', '.join(sorted(VALID_CONFIG_ITEMS))))

    if 'sentence' not in config or ('should_match' not in config and 'should_not_match' not in config):
        raise ValueError('You have to provide a sentence and the function name which should (not) be matched (should_match, should_not_match)')


def output_failure(step_func, errors):
    """
    Write the given errors to stdout.
    """
    sys.stdout.write(u(colorful.bold_red(u'✘')))
    if step_func is not None:
        sys.stdout.write(u(colorful.red(' (at {0})'.format(get_func_location(step_func)))))

    sys.stdout.write('\n')

    for error in errors:
        print(u(colorful.red('  - {0}'.format(error))))


def check_step_arguments(expected_arguments, arguments):
    """
    Check if the given expected arguments
    match the actual arguments
    """
    errors = []
    for arg_name, arg_value in expected_arguments.items():
        if arg_name not in arguments:
            errors.append('Expected argument "{0}" is not in matched arguments {1}'.format(
                arg_name, list(arguments.keys())))
            continue

        use_repr = False

        # check if argument value is a dict, if yes we'll do thorough comparison
        if isinstance(arg_value, dict) and 'type' not in arg_value.keys():
            _type = 'dict'
            value = arg_value
        elif isinstance(arg_value, dict):
            _type = arg_value['type']
            value = arg_value['value']
            # Use repr protocol to match argument values
            use_repr = 'use_repr' in arg_value and arg_value['use_repr']

            # check if value should be casted to the given type
            if 'cast' in arg_value and arg_value['cast'] is True:
                obj_type = locate(_type)
                if obj_type is None:
                    errors.append('Cannot cast to type "{0}" because it is unknown'.format(
                        _type))
                    continue

                try:
                    value = obj_type(value)
                except Exception as exc:
                    errors.append('Failed to cast "{0}" to given type "{1}"'.format(
                        value, type))
                    continue
        else:
            _type = type(arg_value).__name__
            value = arg_value

        if not use_repr and _type != type(value).__name__:
            errors.append('Conflicting argument configuration: given value is actually of type "{0}" although it should match a value of type "{1}"'.format(
                type(value).__name__, _type))
            continue

        if type(arguments[arg_name]).__name__ != _type:
            errors.append('Expected argument "{0}" is of type "{1}" instead "{2}"'.format(
                arg_name, type(arguments[arg_name]).__name__, _type))
            continue

        matched = None
        if use_repr:
            matched = repr(arguments[arg_name]) != value
        else:
            matched = arguments[arg_name] != value

        if matched:
            errors.append('Expected argument "{0}" with value "{1}" does not match value "{2}"'.format(
                arg_name, value, arguments[arg_name]))
    return errors


def merge_step_args(step_func):
    """
    Merges the arguments and keyword arguments
    of the given step function.
    """
    #: Holds the merged arguments as a dict with the corresponding matched values
    args, kwargs = step_func.argument_match.evaluate()
    step_arg_names = get_func_arg_names(step_func.func)[1:]
    arguments = dict(zip(step_arg_names, args))
    arguments.update(kwargs)
    return arguments


def get_missing_steps(steps, covered_steps):
    """
    Get all steps within ``steps`` which are not
    covered by ``covered_steps``.
    """
    missing_steps = []
    for step_func in steps.values():
        if step_func.__name__ not in covered_steps:
            missing_steps.append((step_func.__name__, get_func_location(step_func)))
    return missing_steps



from radish import given, when, then


@when('I have the value {:Boolean}')
def have_value(step, boolean):
    step.context.boolean = boolean


@then('I expect it to be parsed as True')
def expect_true(step):
    assert step.context.boolean is True


@then('I expect it to be parsed as False')
def expect_true(step):
    assert step.context.boolean is False



# -*- coding: utf-8 -*-

from radish import given, when, then
from radish import world

from example.model import Hero


@given('I setup the hero database')
def setup_hero_db(step):
    pass


@when('I query for the hero with name {hero:Hero}')
def query_hero(step, hero):
    step.context.last_queried_hero = hero


@then('I expect it\'s name to be {forename:w} {surname:w}')
def expect_hero(step, forename, surname):
    assert step.context.last_queried_hero.forename == forename
    assert step.context.last_queried_hero.surname == surname



# -*- coding: utf-8 -*-

"""
    This module provides a hook which generates a JUnit XML result file at the end of the run.
"""
from datetime import timedelta
import re

from radish.terrain import world
from radish.hookregistry import after
from radish.exceptions import RadishError
from radish.scenariooutline import ScenarioOutline
from radish.scenarioloop import ScenarioLoop
from radish.stepmodel import Step
from radish.extensionregistry import extension
import radish.utils as utils


@extension
class JUnitXMLWriter(object):
    """
        JUnit XML Writer radish extension
    """
    OPTIONS = [("--junit-xml=<junitxml>", "write JUnit XML result file after run")]
    LOAD_IF = staticmethod(lambda config: config.junit_xml)
    LOAD_PRIORITY = 60

    def __init__(self):
        try:
            from lxml import etree
        except ImportError:
            raise RadishError('if you want to use the JUnit xml writer you have to "pip install radish-bdd[junit-xml]"')

        after.all(self.generate_junit_xml)

    def _get_element_from_model(self, what, model):
        """
            Create a etree.Element from a given model
        """
        from lxml import etree

        # round duration to 10 decimal points, to avoid it being printed in
        # scientific notation
        duration = "%.10f" % model.duration.total_seconds() if model.starttime and model.endtime else ""
        return etree.Element(
            what,
            sentence=model.sentence,
            id=str(model.id),
            result=model.state,
            starttime=utils.datetime_to_str(model.starttime),
            endtime=utils.datetime_to_str(model.endtime),
            duration=duration,
            testfile=model.path
        )

    def _strip_ansi(self, text):
        """
            Strips ANSI modifiers from the given text
        """
        pattern = re.compile(r"(\\033\[\d+(?:;\d+)*m)")
        return pattern.sub("", text)

    def generate_junit_xml(self, features, marker):
        """
            Generates the junit xml
        """
        from lxml import etree
        if not features:
            raise RadishError("No features given to generate JUnit xml file")


        duration = timedelta()
        for feature in features:
            if feature.state in [Step.State.PASSED, Step.State.FAILED]:
                duration += feature.duration

        testsuites_element = etree.Element(
            "testsuites",
            time=str(duration.total_seconds())
        )

        for feature in features:

            if not feature.has_to_run(world.config.scenarios):
                continue

            testsuite_states = {"failures" : 0,
                                "errors" : 0,
                                "skipped" : 0,
                                "tests" : 0}

            for scenario in (s for s in feature.all_scenarios if not isinstance(s, (ScenarioOutline, ScenarioLoop))):
                if not scenario.has_to_run(world.config.scenarios):
                    continue

                testsuite_states["tests"] += 1
                if scenario.state in [Step.State.UNTESTED, Step.State.PENDING, Step.State.SKIPPED]:
                    testsuite_states["skipped"] += 1
                if scenario.state is Step.State.FAILED:
                    testsuite_states["failures"] += 1

            testsuite_element = etree.Element(
                "testsuite",
                name=feature.sentence,
                failures=str(testsuite_states["failures"]),
                errors=str(testsuite_states["errors"]),
                skipped=str(testsuite_states["skipped"]),
                tests=str(testsuite_states["tests"]),
                time=str(feature.duration.total_seconds())
            )

            for scenario in (s for s in feature.all_scenarios if not isinstance(s, (ScenarioOutline, ScenarioLoop))):
                if not scenario.has_to_run(world.config.scenarios):
                    continue

                testcase_element = etree.Element(
                    "testcase",
                    classname=feature.sentence,
                    name=scenario.sentence,
                    time=str(scenario.duration.total_seconds())
                )

                if scenario.state in [Step.State.UNTESTED, Step.State.PENDING, Step.State.SKIPPED]:
                    skipped_element = etree.Element(
                        "skipped"
                    )
                    testcase_element.append(skipped_element)

                if scenario.state is Step.State.FAILED:
                    steps_sentence = []
                    for step in scenario.all_steps:
                        step_element = self._get_element_from_model("step", step)
                        steps_sentence.append(step.sentence)
                        if step.state is Step.State.FAILED:
                            failure_element = etree.Element(
                                "failure",
                                type=step.failure.name,
                                message=step.sentence
                            )
                            failure_element.text = etree.CDATA("%s\n\n%s" % ("\n".join(steps_sentence), self._strip_ansi(step.failure.traceback)))
                            testcase_element.append(failure_element)

                testsuite_element.append(testcase_element)

            testsuites_element.append(testsuite_element)

        with open(world.config.junit_xml, "w+") as f:
            content = etree.tostring(testsuites_element, pretty_print=True, xml_declaration=True, encoding="utf-8")
            try:
                if not isinstance(content, str):
                    content = content.decode("utf-8")
            except Exception:
                pass
            finally:
                f.write(content)



# -*- coding: utf-8 -*-

from radish import step, given, when, then


@step('a user named {:w}')
def have_user(step, username):
    # print(step.context)
    step.context.users[username] = {
        'site': None
    }


@step('a personal site owned by {username:w}')
def have_personal_site(step, username):
    # print(step.context)
    step.context.users[username]['site'] = {
        'access': []
    }


@step('{username:w} grants access to {consumer:w}')
def grant_personal_site_access(step, username, consumer):
    # print(step.context)
    step.context.users[username]['site']['access'].append(consumer)


@when('I\'m logged in as {username:w}')
def logged_in(step, username):
    # print(step.context)
    step.context.current_user = username


@then('I can access {username:w} personal site')
def can_access_personal_site(step, username):
    # print(step.context)
    assert step.context.current_user in step.context.users[username]['site']['access']


@then('I cannot access {username:w} personal site')
def cannot_access_personal_site(step, username):
    # print(step.context)
    assert step.context.current_user not in step.context.users[username]['site']['access']
